#include "IPostProcessingHandler.hpp"


//  SuperTuxKart - a fun racing game with go-kart
//  Copyright (C) 2011 the SuperTuxKart team
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA

#include <IGPUProgrammingServices.h>
#include <IMaterialRendererServices.h>
#include "IPostProcessingHandler.hpp"



using namespace irr;
using namespace video;
using namespace scene;

IPostProcessingHandler::IPostProcessingHandler() :
    m_supported(false)
    , m_render_target(nullptr)
    , Mesh(0)
{
//    m_boost_amount = 0.0f;
}

IPostProcessingHandler::~IPostProcessingHandler()
{
}



//		// Create screen quad shader callback.
//		ScreenQuadCB* SQCB = new ScreenQuadCB(this, true);
//
//		// Light modulate.
//		LightModulate = gpu->addHighLevelShaderMaterial(
//			sPP.ppShader(SCREEN_QUAD_V[shaderExt]).c_str(), "vertexMain", vertexProfile,
//			sPP.ppShader(LIGHT_MODULATE_P[shaderExt]).c_str(), "pixelMain", pixelProfile, SQCB);
//
//		// Simple present.
//		Simple = gpu->addHighLevelShaderMaterial(
//			sPP.ppShader(SCREEN_QUAD_V[shaderExt]).c_str(), "vertexMain", vertexProfile,
//			sPP.ppShader(SIMPLE_P[shaderExt]).c_str(), "pixelMain", pixelProfile, SQCB,
//			video::EMT_TRANSPARENT_ADD_COLOR);
//
//		// VSM blur.
//		VSMBlurH = gpu->addHighLevelShaderMaterial(
//			sPP.ppShader(SCREEN_QUAD_V[shaderExt]).c_str(), "vertexMain", vertexProfile,
//			sPP.ppShader(VSM_BLUR_P[shaderExt]).c_str(), "pixelMain", pixelProfile, SQCB);
//
//		sPP.addShaderDefine("VERTICAL_VSM_BLUR");
//
//		VSMBlurV = gpu->addHighLevelShaderMaterial(
//			sPP.ppShader(SCREEN_QUAD_V[shaderExt]).c_str(), "vertexMain", vertexProfile,
//			sPP.ppShader(VSM_BLUR_P[shaderExt]).c_str(), "pixelMain", pixelProfile, SQCB);
//
//		// Drop the screen quad callback.
//		SQCB->drop();
//


//Nausicaa of The Valley of The Wind [Blu-Ray 720p].part1.rar

/** Initialization */
void IPostProcessingHandler::init(video::IVideoDriver* video_driver)
{
    // Check if post-processing is supported on this hardware
    m_supported = false;
    if( video_driver->queryFeature(video::EVDF_ARB_GLSL) &&
        video_driver->queryFeature(video::EVDF_PIXEL_SHADER_2_0) &&
        video_driver->queryFeature(video::EVDF_RENDER_TO_TARGET))
    {
        m_supported = true;
    }

    // Initialization
    if(m_supported)
    {
        // Render target
//        m_render_target = video_driver->addRenderTargetTexture(video_driver->getScreenSize(), "postprocess");
//        if(!m_render_target)
//        {
//            fprintf(stderr, "Couldn't create the render target for post-processing, disabling it\n");
////            UserConfigParams::m_postprocess_enabled = false;
//        }

        // Material and shaders
        IGPUProgrammingServices* gpu = video_driver->getGPUProgrammingServices();
        s32 material_type = gpu->addHighLevelShaderMaterialFromFiles(
//                    (file_manager->getDataDir() + "shaders/motion_blur.vert").c_str(), "main", video::EVST_VS_2_0,
//                    (file_manager->getDataDir() + "shaders/motion_blur.frag").c_str(), "main", video::EPST_PS_2_0,
                    // File / Entry point / vsCOmpileTarget
                    "shaders/ScreenQuad.glsl", "main", video::EVST_VS_2_0);

//                    (file_manager->getDataDir() + "shaders/motion_blur.frag").c_str(), "main", video::EPST_PS_2_0,
//                    this, video::EMT_SOLID);
        m_material.MaterialType = (E_MATERIAL_TYPE)material_type;
//        m_material.MaterialType = EMT_SOLID  ;

        m_material.setTexture(0, m_render_target);
        m_material.Wireframe = false;
        m_material.Lighting = false;
        m_material.ZWriteEnable = false;
    }


    m_material.MaterialType = EMT_SOLID  ;

    m_material.setTexture(0, m_render_target);
    m_material.Wireframe = false;
    m_material.Lighting = false;
    m_material.ZWriteEnable = false;

	Mesh = new irr::scene:: SMeshBuffer( );
	Mesh->Vertices.reallocate( 4 );
	Mesh->Indices.reallocate( 6 );

	Mesh->Vertices.push_back( irr::video::S3DVertex( -1.0f, -1.0f, 0.0f, 0.0f, 0.0f, 0.0f, irr::video::SColor( 255u, 255u, 255u, 255u ), 0.0f, 0.0f ) );
	Mesh->Vertices.push_back( irr::video::S3DVertex(  1.0f, -1.0f, 0.0f, 0.0f, 0.0f, 0.0f, irr::video::SColor( 255u, 255u, 255u, 255u ), 1.0f, 0.0f ) );
	Mesh->Vertices.push_back( irr::video::S3DVertex( -1.0f,  1.0f, 0.0f, 0.0f, 0.0f, 0.0f, irr::video::SColor( 255u, 255u, 255u, 255u ), 0.0f, 1.0f ) );
	Mesh->Vertices.push_back( irr::video::S3DVertex(  1.0f,  1.0f, 0.0f, 0.0f, 0.0f, 0.0f, irr::video::SColor( 255u, 255u, 255u, 255u ), 1.0f, 1.0f ) );

	Mesh->Indices.push_back( 0u );
	Mesh->Indices.push_back( 2u );
	Mesh->Indices.push_back( 1u );
	Mesh->Indices.push_back( 3u );
	Mesh->Indices.push_back( 1u );
	Mesh->Indices.push_back( 2u );

	Mesh->setHardwareMappingHint( irr::scene::EHM_STATIC, irr::scene::EBT_VERTEX_AND_INDEX );
}



void IPostProcessingHandler::render()
{



    // Draw the fullscreen quad while applying the corresponding post-processing shaders
		irr::video::IVideoDriver* driver = Scene->getVideoDriver( );
//		driver->setRenderTarget( 0, false, false );

        // Target/clearBackBuffer/clearZBuffer
        driver->setRenderTarget(video::ERT_FRAME_BUFFER,false,false);

//        driver->setTransform(irr::video::ETS_WORLD, irr::core::matrix4());


		driver->setMaterial( m_material );
		driver->drawMeshBuffer( Mesh );



//		driver->setRenderTarget(0, true, true, 0);

}





























#if 0

/** Initialization */
void PostProcessing::init(video::IVideoDriver* video_driver)
{
    // Check if post-processing is supported on this hardware
    m_supported = false;
    if( video_driver->queryFeature(video::EVDF_ARB_GLSL) &&
        video_driver->queryFeature(video::EVDF_PIXEL_SHADER_2_0) &&
        video_driver->queryFeature(video::EVDF_RENDER_TO_TARGET))
    {
        m_supported = true;
    }

    // Initialization
    if(m_supported)
    {
        // Render target
        m_render_target = video_driver->addRenderTargetTexture(video_driver->getScreenSize(), "postprocess");
        if(!m_render_target)
        {
            fprintf(stderr, "Couldn't create the render target for post-processing, disabling it\n");
            UserConfigParams::m_postprocess_enabled = false;
        }

        // Material and shaders
        IGPUProgrammingServices* gpu = video_driver->getGPUProgrammingServices();
        s32 material_type = gpu->addHighLevelShaderMaterialFromFiles(
                    (file_manager->getDataDir() + "shaders/motion_blur.vert").c_str(), "main", video::EVST_VS_2_0,
                    (file_manager->getDataDir() + "shaders/motion_blur.frag").c_str(), "main", video::EPST_PS_2_0,
                    this, video::EMT_SOLID);
        m_material.MaterialType = (E_MATERIAL_TYPE)material_type;
        m_material.setTexture(0, m_render_target);
        m_material.Wireframe = false;
        m_material.Lighting = false;
        m_material.ZWriteEnable = false;
    }

	Mesh = new irr::scene::CMeshBuffer<irr::video::S3DVertex>( );
	Mesh->Vertices.reallocate( 4 );
	Mesh->Indices.reallocate( 6 );

	Mesh->Vertices.push_back( irr::video::S3DVertex( -1.0f, -1.0f, 0.0f, 0.0f, 0.0f, 0.0f, irr::video::SColor( 255u, 255u, 255u, 255u ), 0.0f, 0.0f ) );
	Mesh->Vertices.push_back( irr::video::S3DVertex(  1.0f, -1.0f, 0.0f, 0.0f, 0.0f, 0.0f, irr::video::SColor( 255u, 255u, 255u, 255u ), 1.0f, 0.0f ) );
	Mesh->Vertices.push_back( irr::video::S3DVertex( -1.0f,  1.0f, 0.0f, 0.0f, 0.0f, 0.0f, irr::video::SColor( 255u, 255u, 255u, 255u ), 0.0f, 1.0f ) );
	Mesh->Vertices.push_back( irr::video::S3DVertex(  1.0f,  1.0f, 0.0f, 0.0f, 0.0f, 0.0f, irr::video::SColor( 255u, 255u, 255u, 255u ), 1.0f, 1.0f ) );

	Mesh->Indices.push_back( 0u );
	Mesh->Indices.push_back( 2u );
	Mesh->Indices.push_back( 1u );
	Mesh->Indices.push_back( 3u );
	Mesh->Indices.push_back( 1u );
	Mesh->Indices.push_back( 2u );


}



/** Setup the render target */
void PostProcessing::beginCapture()
{
    if(!m_supported || !UserConfigParams::m_postprocess_enabled ||
       race_manager->getNumPlayers() > 1)
        return;

    // don't capture the input when we have no post-processing to add
    // it will be faster and this ay we won't lose anti-aliasing
    if (m_boost_amount <= 0.0f)
    {
        m_used_pp_this_frame = false;
        return;
    }

    m_used_pp_this_frame = true;
    irr_driver->getVideoDriver()->setRenderTarget(m_render_target, true, true);
}

/** Restore the framebuffer render target */
void PostProcessing::endCapture()
{
    if(!m_supported || !UserConfigParams::m_postprocess_enabled ||
       race_manager->getNumPlayers() > 1)
        return;

    if (m_used_pp_this_frame)
    {
        //Render target is the main color frame buffer.
        irr_driver->getVideoDriver()->setRenderTarget(video::ERT_FRAME_BUFFER, true, true, 0);
    }
}

void PostProcessing::update(float dt)
{
    if (m_boost_amount > 0.0f)
    {
        m_boost_amount -= dt*3.5f;
        if (m_boost_amount < 0.0f) m_boost_amount = 0.0f;
    }
}

/** Render the post-processed scene */
void PostProcessing::render()
{
//    if(!m_supported || !UserConfigParams::m_postprocess_enabled ||
//       race_manager->getNumPlayers() > 1)
//        return;
//
//    if (!m_used_pp_this_frame)
//    {
//        return;
//    }

    // Draw the fullscreen quad while applying the corresponding post-processing shaders
    video::IVideoDriver*    video_driver = irr_driver->getVideoDriver();
    /*video::S3DVertex        vertices[6];

    video::SColor white(0xFF, 0xFF, 0xFF, 0xFF);
    vertices[0] = irr::video::S3DVertex(-1.0f,-1.0f,0.0f,0,0,1, white, 0.0f,1.0f);
    vertices[1] = irr::video::S3DVertex(-1.0f, 1.0f,0.0f,0,0,1, white, 0.0f,0.0f);
    vertices[2] = irr::video::S3DVertex( 1.0f, 1.0f,0.0f,0,0,1, white, 1.0f,0.0f);
    vertices[3] = irr::video::S3DVertex( 1.0f,-1.0f,0.0f,0,0,1, white, 1.0f,1.0f);
    vertices[4] = irr::video::S3DVertex(-1.0f,-1.0f,0.0f,0,0,1, white, 0.0f,1.0f);
    vertices[5] = irr::video::S3DVertex( 1.0f, 1.0f,0.0f,0,0,1, white, 1.0f,0.0f);

    u16 indices[6] = {0, 1, 2, 3, 4, 5};

    video_driver->setMaterial(m_material);
    video_driver->drawIndexedTriangleList(&vertices[0], 6, &indices[0], 2);
    */

		driver->setRenderTarget( target, false, false );
		driver->setMaterial( Tmats[ i ] );
		driver->drawMeshBuffer( Mesh );
}


/** Implement IShaderConstantsSetCallback. Shader constants setter for post-processing */
//void PostProcessing::OnSetConstants(video::IMaterialRendererServices *services, s32 user_data)
//{
//    services->setPixelShaderConstant("boost_amount", &m_boost_amount, 1);
//    const float texunit = 0.0f;
//    services->setPixelShaderConstant("color_buffer", &texunit, 1);  // Irrlicht knows this is actually a GLint and makes
//                                                                   // the conversion
//}

#endif
